# -*- coding: utf-8 -*-
# $Id: sv.py 8006 2016-12-22 23:02:44Z milde $
# Author: Adam Chodorowski <chodorowski@users.sourceforge.net>
# Copyright: This module has been placed in the public domain.

# New language mappings are welcome.  Before doing a new translation, please
# read <http://docutils.sf.net/docs/howto/i18n.html>.  Two files must be
# translated for each language: one in docutils/languages, the other in
# docutils/parsers/rst/languages.

"""
Swedish language mappings for language-dependent features of Docutils.
"""

__docformat__ = 'reStructuredText'

labels = {
    'author':       u'Författare',
    'authors':      u'Författare',
    'organization': u'Organisation',
    'address':      u'Adress',
    'contact':      u'Kontakt',
    'version':      u'Version',
    'revision':     u'Revision',
    'status':       u'Status',
    'date':         u'Datum',
    'copyright':    u'Copyright',
    'dedication':   u'Dedikation',
    'abstract':     u'Sammanfattning',
    'attention':    u'Observera!',
    'caution':      u'Akta!', # 'Varning' already used for 'warning'
    'danger':       u'FARA!',
    'error':        u'Fel',
    'hint':         u'Vink',
    'important':    u'Viktigt',
    'note':         u'Notera',
    'tip':          u'Tips',
    'warning':      u'Varning',
    'contents':     u'Innehåll' }
"""Mapping of node class name to label text."""

bibliographic_fields = {
    # 'Author' and 'Authors' identical in Swedish; assume the plural:
    u'författare': 'authors',
    u' n/a':            'author',
    u'organisation':    'organization',
    u'adress':          'address',
    u'kontakt':         'contact',
    u'version':         'version',
    u'revision':        'revision',
    u'status':          'status',
    u'datum':           'date',
    u'copyright':       'copyright',
    u'dedikation':      'dedication', 
    u'sammanfattning':  'abstract' }
"""Swedish (lowcased) to canonical name mapping for bibliographic fields."""

author_separators = [';', ',']
"""List of separator strings for the 'Authors' bibliographic field. Tried in
order."""



from functools import partial
from threading import local

import util
from schema import GrofersRequestLogSchemaAdapter

_local = local()

_logger_factory_cls = util.StructlogFactoryImpl

_schema_builder = None


def configure(schema_builder=GrofersRequestLogSchemaAdapter, **cfg):
    """
    One time setup for the lifetime of an application.
    :param schema_builder: dictifiable object
    :param cfg: kwargs that are accepted by logging.basicConfig
    """
    global _schema_builder
    util.configure(**cfg)

    if schema_builder:
        _schema_builder = schema_builder


def new_logger(**kw):
    """
    Setup once every session or request cycle.
    :param kw: kwargs bound to logger for scope of session.
    """
    global _local
    _local.logger = _logger_factory_cls()
    bind(**kw)


def get_trace_id():
    if not hasattr(_local, 'trace_id'):
        return None
    return _local.trace_id


def _set_trace_id(trace_id):
    global _local
    _local.trace_id = trace_id


def bind(request=None, **kwargs):
    global _local
    global _schema_builder

    if not hasattr(_local, 'logger'):
        raise RuntimeError("bind_logger() invoked before new_logger() invocation")

    schema = kwargs or {}
    if _schema_builder:
        schema_builder = _schema_builder(request, **schema)
        schema_dict = schema_builder.as_dict()
        _set_trace_id(schema_dict.get('gr_trace_id'))
        schema.update(schema_dict)

    _local.logger.bind(**schema)


def _get_logger(*args, **kwargs):
    global _local
    if not hasattr(_local, 'logger'):
        _local.logger = _logger_factory_cls(*args, **kwargs)

    kwargs = kwargs or {}
    return _local.logger.get_logger(*args, **kwargs)


def _log_supplier(*args, **kwargs):
    """
    Acts as a generator of the `logger` object when
    requested. This does the necessary *magic* to fetch the
    available request context.
    @param kwargs: initial values for the structlog
    @return:
    """
    return _get_logger(*args, **kwargs)


def get_logger(*args, **kwargs):
    """
    Returns a proxy to the logger object. This will ensure that
    even if the logger has been declared at the module level, it will
    still have access to the request bound context, if present.
    This will ensure that the common pattern like::

        from glogger import get_logger

        log = get_logger()

        class SomeController:
            def some_method():
                log.info('Some method of some controllers invoked')
                # the log line published above will have any
                # request bound context

    continue to work.
    @param kwargs: any initial values needed to be passed to `structlog`
    @return: proxy to the actual logger object
    """
    return util.Proxy(partial(_log_supplier, *args, **kwargs))


# CamelCase alias for `glogger.get_logger`
getLogger = get_logger
new_bounded_logger = new_logger



import sys

import abc
import functools
import uuid

import config
from _setting import Settings

settings = Settings()

# compatible with Python 2 *and* 3:
ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})


def configure(**kwargs):
    global settings
    settings = Settings(**kwargs)
    if settings.get_env() == 'dev':
        config.development_cfg(**kwargs)
    else:
        config.production_cfg(**kwargs)


class LoggerFactory(ABC):
    @abc.abstractmethod
    def bind(self, **kw):
        pass

    @abc.abstractmethod
    def get_logger(self, *args, **kw):
        """Returns a new logger bounded with the given context"""
        pass


try:
    import structlog
except ImportError as e:
    raise e


def generate_trace_id():
    return "{0}={1}".format(str(settings.get_app()), str(uuid.uuid1()))


class StructlogFactoryImpl(LoggerFactory):
    _logger = None

    def __init__(self, *args, **kw):
        self._logger = structlog.get_logger(*args).new(**kw)

    def bind(self, **kw):
        self._logger = self._logger.bind(**kw)

    def get_logger(self, *args, **kw):
        # Bind module specific keys with the fresh values.
        # I mean, who does't like fresh!
        if 'module' not in kw and args:
            kw['module'] = args[0]

        return self._logger.bind(**kw)


def compose(*functions):
    """
    Composes n functions. Composition of two functions `f` and `g`
    would be `f(g(x))`. Similarly, for *n* functions, f1, f2, ..., fn,
    the composition would be: fn(fn-1(...f2(f1(x))...)) for input `x`.
    @param functions: list of functions to be composed
    @return: the composed function
    """
    return functools.reduce(lambda f, g: lambda x: f(g(x)), functions, lambda x: x)


class Proxy(object):
    def __init__(self, delegate):
        """
        @param delegate: the object for which this class
        acts as a proxy. It could be an object or a callable
        """
        self._delegate = delegate

    def __getattr__(self, name):
        """
        Fetches the desired attribute from the delegate
        @param name: name of the attribute
        @return: the requested attribute if present, else `None`
        """
        self._before_advice()
        return getattr(self._get_delegate(), name, None)

    def _get_delegate(self):
        """
        Returns the delegated object. If the object is callable,
        it invokes the function to get the delegated object
        @return: the delegated object
        """
        if callable(self._delegate):
            return self._delegate()
        return self._delegate

    def _before_advice(self):
        pass


def find_caller():
    """
    Finds the first caller frame outside of glogger so that the caller
    info is populated for wrapping stdlib.
    """
    f, _ = _find_first_app_frame_and_name(['logging'])
    return f.f_code.co_filename, f.f_lineno, f.f_code.co_name


def _find_first_app_frame_and_name(additional_ignores=None):
    """
    Remove all intra-glogger calls and return the relevant app frame.

    :param additional_ignores: Additional names with which the first frame must
        not start.
    :type additional_ignores: `list` of `str` or `None`

    :rtype: tuple of (frame, name)
    """
    ignores = ["glogger"] + (additional_ignores or [])
    f = sys._getframe()
    name = f.f_globals.get("__name__") or "?"
    while any(name.startswith(i) for i in ignores):
        if f.f_back is None:
            name = "?"
            break
        f = f.f_back
        name = f.f_globals.get("__name__") or "?"
    return f, name



"""Module containing logic for handling settings."""
# Copyright 2018 Ian Stapleton Cordasco
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

from twine import exceptions
from twine import repository
from twine import utils


class Settings(object):
    """Object that manages the configuration for Twine.

    This object can only be instantiated with keyword arguments.

    For example,

    .. code-block:: python

        Settings(True, username='fakeusername')

    Will raise a :class:`TypeError`. Instead, you would want

    .. code-block:: python

        Settings(sign=True, username='fakeusername')
    """

    @utils.no_positional(allow_self=True)
    def __init__(self,
                 sign=False, sign_with='gpg', identity=None,
                 username=None, password=None,
                 comment=None,
                 config_file='~/.pypirc', skip_existing=False,
                 cacert=None, client_cert=None,
                 repository_name='pypi', repository_url=None,
                 verbose=False,
                 disable_progress_bar=False,
                 **ignored_kwargs
                 ):
        """Initialize our settings instance.

        :param bool sign:
            Configure whether the package file should be signed.

            This defaults to ``False``.
        :param str sign_with:
            The name of the executable used to sign the package with.

            This defaults to ``gpg``.
        :param str identity:
            The GPG identity that should be used to sign the package file.
        :param str username:
            The username used to authenticate to the repository (package
            index).
        :param str password:
            The password used to authenticate to the repository (package
            index).
        :param str comment:
            The comment to include with each distribution file.
        :param str config_file:
            The path to the configuration file to use.

            This defaults to ``~/.pypirc``.
        :param bool skip_existing:
            Specify whether twine should continue uploading files if one
            of them already exists. This primarily supports PyPI. Other
            package indexes may not be supported.

            This defaults to ``False``.
        :param str cacert:
            The path to the bundle of certificates used to verify the TLS
            connection to the package index.
        :param str client_cert:
            The path to the client certificate used to perform authentication
            to the index.

            This must be a single file that contains both the private key and
            the PEM-encoded certificate.
        :param str repository_name:
            The name of the repository (package index) to interact with. This
            should correspond to a section in the config file.
        :param str repository_url:
            The URL of the repository (package index) to interact with. This
            will override the settings inferred from ``repository_name``.
        :param bool verbose:
            Show verbose output.
        :param bool disable_progress_bar:
            Disable the progress bar.

            This defaults to ``False``
        """
        self.config_file = config_file
        self.comment = comment
        self.verbose = verbose
        self.disable_progress_bar = disable_progress_bar
        self.skip_existing = skip_existing
        self._handle_repository_options(
            repository_name=repository_name, repository_url=repository_url,
        )
        self._handle_package_signing(
            sign=sign, sign_with=sign_with, identity=identity,
        )
        # The following two rely on the parsed repository config
        self._handle_certificates(cacert, client_cert)
        self._handle_authentication(username, password)

    @staticmethod
    def register_argparse_arguments(parser):
        """Register the arguments for argparse."""
        parser.add_argument(
            "-r", "--repository",
            action=utils.EnvironmentDefault,
            env="TWINE_REPOSITORY",
            default="pypi",
            help="The repository (package index) to upload the package to. "
                 "Should be a section in the config file (default: "
                 "%(default)s). (Can also be set via %(env)s environment "
                 "variable.)",
        )
        parser.add_argument(
            "--repository-url",
            action=utils.EnvironmentDefault,
            env="TWINE_REPOSITORY_URL",
            default=None,
            required=False,
            help="The repository (package index) URL to upload the package to."
                 " This overrides --repository. "
                 "(Can also be set via %(env)s environment variable.)"
        )
        parser.add_argument(
            "-s", "--sign",
            action="store_true",
            default=False,
            help="Sign files to upload using GPG.",
        )
        parser.add_argument(
            "--sign-with",
            default="gpg",
            help="GPG program used to sign uploads (default: %(default)s).",
        )
        parser.add_argument(
            "-i", "--identity",
            help="GPG identity used to sign files.",
        )
        parser.add_argument(
            "-u", "--username",
            action=utils.EnvironmentDefault,
            env="TWINE_USERNAME",
            required=False,
            help="The username to authenticate to the repository "
                 "(package index) as. (Can also be set via "
                 "%(env)s environment variable.)",
        )
        parser.add_argument(
            "-p", "--password",
            action=utils.EnvironmentDefault,
            env="TWINE_PASSWORD",
            required=False,
            help="The password to authenticate to the repository "
                 "(package index) with. (Can also be set via "
                 "%(env)s environment variable.)",
        )
        parser.add_argument(
            "-c", "--comment",
            help="The comment to include with the distribution file.",
        )
        parser.add_argument(
            "--config-file",
            default="~/.pypirc",
            help="The .pypirc config file to use.",
        )
        parser.add_argument(
            "--skip-existing",
            default=False,
            action="store_true",
            help="Continue uploading files if one already exists. (Only valid "
                 "when uploading to PyPI. Other implementations may not "
                 "support this.)",
        )
        parser.add_argument(
            "--cert",
            action=utils.EnvironmentDefault,
            env="TWINE_CERT",
            default=None,
            required=False,
            metavar="path",
            help="Path to alternate CA bundle (can also be set via %(env)s "
                 "environment variable).",
        )
        parser.add_argument(
            "--client-cert",
            metavar="path",
            help="Path to SSL client certificate, a single file containing the"
                 " private key and the certificate in PEM format.",
        )
        parser.add_argument(
            "--verbose",
            default=False,
            required=False,
            action="store_true",
            help="Show verbose output."
        )
        parser.add_argument(
            "--disable-progress-bar",
            default=False,
            required=False,
            action="store_true",
            help="Disable the progress bar."
        )

    @classmethod
    def from_argparse(cls, args):
        """Generate the Settings from parsed arguments."""
        settings = vars(args)
        settings['repository_name'] = settings.pop('repository')
        settings['cacert'] = settings.pop('cert')
        return cls(**settings)

    def _handle_package_signing(self, sign, sign_with, identity):
        if not sign and identity:
            raise exceptions.InvalidSigningConfiguration(
                "sign must be given along with identity"
            )
        self.sign = sign
        self.sign_with = sign_with
        self.identity = identity

    def _handle_repository_options(self, repository_name, repository_url):
        self.repository_config = utils.get_repository_from_config(
            self.config_file,
            repository_name,
            repository_url,
        )
        self.repository_config['repository'] = utils.normalize_repository_url(
            self.repository_config['repository'],
        )

    def _handle_authentication(self, username, password):
        self.username = utils.get_username(
            self.repository_config['repository'],
            username,
            self.repository_config
        )
        self.password = utils.get_password(
            self.repository_config['repository'],
            self.username,
            password,
            self.repository_config,
        )

    def _handle_certificates(self, cacert, client_cert):
        self.cacert = utils.get_cacert(cacert, self.repository_config)
        self.client_cert = utils.get_clientcert(
            client_cert,
            self.repository_config,
        )

    def check_repository_url(self):
        """Verify we are not using legacy PyPI.

        :raises:
            :class:`~twine.exceptions.UploadToDeprecatedPyPIDetected`
        """
        repository_url = self.repository_config['repository']

        if repository_url.startswith((repository.LEGACY_PYPI,
                                      repository.LEGACY_TEST_PYPI)):
            raise exceptions.UploadToDeprecatedPyPIDetected.from_args(
                repository_url,
                utils.DEFAULT_REPOSITORY,
                utils.TEST_REPOSITORY
            )

    def create_repository(self):
        """Create a new repository for uploading."""
        repo = repository.Repository(
            self.repository_config['repository'],
            self.username,
            self.password,
            self.disable_progress_bar
        )
        repo.set_certificate_authority(self.cacert)
        repo.set_client_certificate(self.client_cert)
        return repo



import logging

import glogger


class DjangoMiddleware(object):
    """
    Old middleware. Before middleware interface was changed.
    """

    def __init__(self):
        from django.conf import settings as django_settings
        glogger.configure(
            app=getattr(django_settings, 'GLOGGER_APP', None) or 'python',
            env=getattr(django_settings, 'GLOGGER_ENV', None) or 'dev',
            level=getattr(django_settings, 'GLOGGER_LEVEL', None) or logging.INFO
        )

    def process_request(self, request):
        glogger.new_logger(request=request)
        request.gr_trace_id = glogger.get_trace_id()

    def process_response(self, request, response):
        # log response too?
        if not response.has_header('X-Gr-Trace-Id'):
            response['X-Gr-Trace-Id'] = glogger.get_trace_id()
        return response

    def process_exception(self, request, exception):
        # log response too?
        pass


class DjangoMiddlewareNew:
    """
    Middleware for latest Django
    """

    def __init__(self, get_response):
        self.get_response = get_response

        from django.conf import settings as django_settings
        glogger.configure(
            app=getattr(django_settings, 'GLOGGER_APP', None) or 'python',
            env=getattr(django_settings, 'GLOGGER_ENV', None) or 'dev',
            level=getattr(django_settings, 'GLOGGER_LEVEL', None) or logging.INFO
        )
        # One-time configuration and initialization.

    def __call__(self, request):
        glogger.new_logger(request=request)
        request.gr_trace_id = glogger.get_trace_id()
        response = self.get_response(request)
        return response



from flask import Flask
import pybreaker
import statsd
from datetime import datetime, timedelta
app = Flask(__name__)
client = statsd.StatsClient('localhost', '8125')
db_breaker = pybreaker.CircuitBreaker(client, reset_timeout=5, buffer_reset_flag=True
                                      )

opened_at = datetime.utcnow()
timeout = timedelta(seconds=10)


@app.route('/')
@db_breaker
def hello_word():
    time_now = datetime.utcnow()
    while time_now < (opened_at + timeout):
        raise NotImplementedError()
    return "Total : {0}\nFail : {1}\nSuccess : {2}\nState : {3}" \
        .format(db_breaker._state_storage._total_calls, db_breaker._state_storage.fail_counter,
                db_breaker._state_storage.success_counter, db_breaker.current_state)




@app.route('/err')
@db_breaker
def test():
    print db_breaker.current_state
    raise NotImplementedError()


if __name__ == '__main__':
    app.run(debug=True, host='127.0.0.1', port=8082)

